#ifndef WEAK_CLASSIFIER_H
#define WEAK_CLASSIFIER_H 1

#include "classifier.h"
#include <iostream>
#include <vector>

namespace Classifier
{
	//
	// weak classifier
	//
	template <typename CATEGORY, typename DATA>
	class weak_classifier :	public classifier<CATEGORY, DATA>
	{
	public:
		weak_classifier(double random_noise, double random_noise_threshold):
		  random_noise__(random_noise), random_noise_threshold__(random_noise_threshold) {};
		  ~weak_classifier(void){};

		  //
		  // In this toy example category is given by the parity of the vector
		  // there is a noise introduced by the random_noise__ parameter
		  //
		  bool classify(std::vector<int>& d)
		  {
			  bool return_value = oracle(d);

			  if (random_noise__ < random_noise_threshold__)
				  return_value = (!return_value); // a mistake

			  return return_value;
		  }

		  //  test function
		  // 
		  bool test(std::vector<int>& d)
		  {
			  return oracle(d);
		  }

	private:
		// random noise
		double random_noise__;
		double random_noise_threshold__;

		// toy example
		// this oracle function makes always a perfect classification
		bool oracle(std::vector<int>& d)
		{
			std::vector<int>::const_iterator dit = d.begin();
			std::vector<int>::const_iterator dend = d.end();
			int v = 0;
			for (; dit != dend; dit++)
				v += *dit;
			return (v % 2) ? true : false;
		}
	};

};
#endif